# Copyright [2021-2025] Thanh Nguyen
# Copyright [2022-2023] [CNRS, Toward SAS]

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# flake8: noqa
# isort: skip_file

from .results_manager import ResultsManager
from .error_handling import (
    FigarohExampleError,
    CalibrationError,
    IdentificationError,
    handle_calibration_errors,
    handle_identification_errors,
    validate_input_data,
    setup_example_logging
)

__all__ = [
    'ResultsManager',
    'FigarohExampleError',
    'CalibrationError', 
    'IdentificationError',
    'handle_calibration_errors',
    'handle_identification_errors',
    'validate_input_data',
    'setup_example_logging'
]
