# Copyright [2021-2025] Thanh Nguyen
"""
figaroh: Robot and Human Identification Tools
"""

from . import tools
from . import calibration
from . import identification
from . import measurements
from . import utils
from . import visualisation
from . import optimal


__version__ = "0.1.0"
